<?php
// 環境定義ファイル・稼働モード時の動作を設定
require_once (dirname(__FILE__).'/common.php');

//セッション名を指定
session_name(SESSION_NAME);
//セッション開始
session_start();

//データベース関数のファイルの読み込み
require_once ('../'.L_BASE.'/db/meibo.php');
//検証及びエスケープ処理の関数のファイルの読み込み
require_once 'session_validation.php';

//セッションIDを初期化
$cookie_session_id = NULL;

//クッキーからセッションIDを取得
if (isset($_COOKIE[session_name()])){
 $cookie_session_id = h($_COOKIE[session_name()]) ;
}

//セッションIDを変更（セッションハイジャック対策）
session_regenerate_id(TRUE); 

// 表示メッセージ
$message = "";


//--------------------------------------------------------
// ログインしている場合は強制的にトップページへ
//--------------------------------------------------------

if (isset($cookie_session_id)) {
 try {
  // TBL_USERを読み込む
  $meibo = new meiboDb();
  $ret = $meibo->selectSession($cookie_session_id);
  $meibo->closeDetaBase();

  if (isset($ret[0]) && !empty($ret[0]) && (count($ret) > 0) && isset($_SESSION['user_id'])) { // DBにセッション有
   // セッション比較
   if ($_SESSION['user_id'] == $ret[0]['user_id']) {
    // DBのセッションID同一でユーザー同一なら
    // セッションIDを更新
    $session_id = session_id();
    $meibo = new meiboDb();
    $ret = $meibo->updateSession( session_id(), $_SESSION['user_id'] );
    $meibo->commit();
    $meibo->closeDetaBase();

    // トップページへ
    header("Location: administration", true , 302);
    exit();
   }
  }

 }catch(ErrorException $ex){
  if(DEBUG_LEVEL != "0"){
   error_log("[". date('Y-m-d H:i:s') . "]". "login01\n","3","debug.log");
   error_log(print_r($ex->getMessage(),true)."\n","3","debug.log");
  }
 }catch(PDOException $ex){
  if(DEBUG_LEVEL != "0"){
   error_log("[". date('Y-m-d H:i:s') . "]". "login02\n","3","debug.log");
   error_log(print_r($ex->getMessage(),true)."\n","3","debug.log");
  }
 }
}

//--------------------------------------------------------
// ログイン入力処理
//--------------------------------------------------------

//CSRF対策の固定トークンを生成
if(!isset($_SESSION['ticket'])){
 //セッション変数にトークンを代入
 $_SESSION['ticket'] = sha1(uniqid(mt_rand(), TRUE));
}
 
//変数 $ticket にトークンの値を代入
$ticket = $_SESSION['ticket'];

//入力されて来た場合
if(count($_POST) > 0) {
 //POSTされたデータをチェック
 $_POST = checkInput($_POST);

 //固定トークンを確認（CSRF対策）
 if(isset($_POST['ticket'], $_SESSION['ticket'])){
  $ticket = $_POST['ticket'];
  if($ticket !== $_SESSION['ticket']){
   die('不正な入力です。');
  }
 }else{
  die('不正な入力です。');
 }

 $message = "";
//--------------------------------------------------------
// ログイン失敗処理
//--------------------------------------------------------
 //ユーザー名またはパスワードが送信されて来なかった場合
 if(empty($_POST["uname"]) || empty($_POST["pass"])) {
  $message = "ユーザーIDとパスワードを入力してください";
 }
 //ユーザー名とパスワードが送信されて来た場合
 else {
  //post送信されてきたユーザー名がデータベースにあるか検索
  try {
   $meibo = new meiboDb();
   $ret = $meibo->selectMeibo($_POST["uname"]);
   $meibo->closeDetaBase();
  }
  catch (PDOExeption $e) {
   die('データベースエラー');
  }

  //検索したユーザー名が登録なし
  if (!isset($ret[0]) || empty($ret[0]) || (count($ret) <= 0)) {
   // DBにユーザー無し
   $message="ユーザー名かパスワードが違います。 エラー：001";
  }
  //検索したユーザー名に対してパスワードが正しいかを検証
  //正しくないとき
  else if (!password_verify($_POST['pass'], $ret[0]['password'])) {
   if (isset($ret[0]['user_id']) && !empty($ret[0]['user_id'])) { // DBにユーザー有り
    if($ret[0]['error_cnt'] < 2) {
     // エラー回数＋１
     $meibo = new meiboDb();
     $ret = $meibo->updateError( $ret[0]['user_id'] );
     $meibo->commit();
     $meibo->closeDetaBase();
     // DBにユーザー有り　パスワードエラー
     $message="ユーザー名かパスワードが違います。 エラー：002";
    }
    else {
     $meibo = new meiboDb();
     $ret = $meibo->updateExistence( $ret[0]['user_id'] ,1);
     $meibo->commit();
     $meibo->closeDetaBase();
      $message="アカウントを停止しました";
    }
   }
   else {
    // DBにユーザーなし
    $message="ユーザー名かパスワードが違います。 エラー：003";
   }
  }
  //アカウント停止・削除
  else if ($ret[0]['existence'] != 0)  {
   // パスワードOKもアカウント停止中か削除
   $message="ユーザー名かパスワードが違います。 エラー：004";
  }
  //正しいとき
  else {
//--------------------------------------------------------
// ログイン成功処理
//--------------------------------------------------------
   // セッションにユーザーIDを保存
   $_SESSION['user_id'] = $ret[0]['user_id'] ;
   unset($_SESSION['ticket']);

   // セッションIDを更新
   $session_id = session_id();
   $meibo = new meiboDb();
   $ret = $meibo->updateSession( $session_id, $_SESSION['user_id'] );
   $meibo->commit();
   $meibo->closeDetaBase();

   // トップページへ
   header("Location: administration", true , 302);
   exit();
  }
 }
}

$message = h($message);

// ページ名
define('PAGE_NAME', 'ログイン');

?>

<!DOCTYPE html>
<html lang="ja">
<head>
 <meta charset="UTF-8">
 <title><?php echo P_TITLE;?> | <?php echo PAGE_NAME;?></title>
 <meta http-equiv="X-UA-Compatible" content="IE=edge">
 <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
 <link href="css/bootstrap.min.css" rel="stylesheet">
 <link href="css/style.css" rel="stylesheet">
</head>
<body>

 <?php include 'inc/header.php'; ?> <!-- ヘッダー共通部 -->

 <div class="container-fluid">
  <div class="row">
   <div class="col-sm-0 col-md-1" style="background-color:white;"></div>
   <div class="col-sm-12 col-md-10" style="background-color:white;">
    <br>
    <br>
    <form action="login" method="post" class="form-horizontal">
     <fieldset>
      <legend>ログインページ</legend>
      <div class="form-group">
       <div class="col-lg-10">
        <input type="text" class="form-control" name="uname" id="uname" placeholder="ユーザー名"/>
       </div>
      </div>
      <div class="form-group">
       <div class="col-lg-10">
        <input type="password" class="form-control" name="pass" id="pass" placeholder="パスワード"/>
        <span class="field-icon">
         <i toggle="#password-field" class="glyphicon glyphicon-eye-open toggle-password"></i>
        </span>
        <span class="help-block"><?php echo $message;?></span>
       </div>
      </div>
      <div class="form-group">
       <div class="col-lg-10 col-lg-offset-2">
        <button type="reset" class="btn btn-default">取消</button>
        <button type="submit" class="btn btn-primary">ログイン</button>
       </div>
      </div>
      <input type="hidden" name="ticket" value="<?php echo $ticket; ?>"><br>
     </fieldset>
    </form>
   </div>
   <div class="col-sm-0 col-md-1" style="background-color:white;"></div>
  </div>
 </div>

 <?php include 'inc/footer.php'; ?> <!-- フッター共通部 -->

 <script type="text/javascript" src="js/jquery-3.5.1.js"></script>
 <script type="text/javascript" src="js/bootstrap.min.js"></script>
 <script type="text/javascript" src="js/main.js"></script>

</body>
</html>