<?php
/*
 * ：アンケートページへのアクセスログを管理するクラス
 * @親クラス  ：  ・pdodb
 */
require_once (dirname(__FILE__).'/pdoDb.class.php');
require_once (dirname(__FILE__).'/../setting.php');
require_once (dirname(__FILE__).'/dbconfig.php');

class accesslogDb extends pdodb
{
 const PATTERN = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx';

 /*
  * @countAccessLog（TBL_ACCESSLOGテーブルのデータ数を抽出する）
  * @引数：a_id　アンケートID
  * @引数：gekijo_id　劇場ID
  * @引数：kind　回答属性
  * @返値：データ数
 */
 public function countAccessLog($a_id, $gekijo_id, $kind = null)
 {
  //ＳＱＬ設定
  $sql = '';
  $sql .= 'SELECT count(*) ';
  $sql .= 'FROM '.TBL_ACCESSLOG.' ';
  $sql .= "WHERE  gekijo_id = :gekijo_id ";
  $sql .= "AND  a_id = :a_id ";

  //パラメータの有無を判断（あればパラメータをバインド変数として実行）
  if ( $kind != null && !empty( $kind ) ){
   $sql .= "AND  kind = :kind";
   $params = array(
    'gekijo_id' => $gekijo_id ,
    'a_id' => $a_id ,
    'kind' => $kind
   );
  }
  else {
   $params = array(
    'gekijo_id' => $gekijo_id ,
    'a_id' => $a_id
   );
  }
  $stmt = parent::execute($sql, $params);

  //ＳＱＬを実行及び返値をこのメソッドの返値として設定
  $ret = $stmt->fetchAll();
  return $ret[0][0];
 }


 /*
  * @selectAccessLog（TBL_ACCESSLOGテーブルのデータを抽出する）
  * @引数：parm　クッキーデータ
  * @返値：抽出データ
  */
 public function selectAccessLog($parm = null)
 {
  //ＳＱＬ設定
  $sql = '';
  $sql .= 'SELECT * ';
  $sql .= 'FROM '.TBL_ACCESSLOG.' ';

  //パラメータの有無を判断（あればパラメータをバインド変数として実行）
  if ( $parm != null && !empty( $parm ) ){
   $sql .= "WHERE  cookieData = :cookieData";
   $params = array(
    'cookieData' => $parm
   );
   $stmt = parent::execute($sql, $params);
  }
  else {
   $stmt = parent::execute($sql);
  }

  //ＳＱＬを実行及び返値をこのメソッドの返値として設定
  $ret = $stmt->fetchAll();
  return $ret;
 }

 /*
  * @insertAccessLog（TBL_ACCESSLOGテーブルにデータを追加する）
  * @引数：a_id　アンケートID
  * @引数：gekijo_id　劇場ID
  * @引数：r_id　アンケート場所
  * @引数：cookieData　クッキーデータ
  * @返値：実行結果
  */
 public function insertAccessLog($a_id, $gekijo_id, $r_id, $cookieData)
 {
  //更新日時
  $mynowtime = date ("Y-m-d H:i:s");
  $getBrowser = get_browser(null, true);
  $remoteAddr = $this->realaddr();

  //ＳＱＬ設定
  $sql = '';
  $sql .= 'INSERT INTO '.TBL_ACCESSLOG.' ';
  $sql .= "( ";
  $sql .= ' requestUri ';
  $sql .= ', referer ';
  $sql .= ', remoteAddr ';
  $sql .= ', remoteHost ';
  $sql .= ', browser ';
  $sql .= ', version ';
  $sql .= ', platform ';
  $sql .= ', userAgent ';
  $sql .= ', gekijo_id ';
  $sql .= ', a_id ';
  $sql .= ', r_id ';
  $sql .= ', cookieData ';
  $sql .= ', creation_dt ';
  $sql .= ', modification_dt ';
  $sql .= ") ";
  $sql .= "VALUES ";
  $sql .= "( ";
  $sql .= ":requestUri ";
  $sql .= ",:referer ";
  $sql .= ",:remoteAddr ";
  $sql .= ",:remoteHost ";
  $sql .= ",:browser ";
  $sql .= ",:version ";
  $sql .= ",:platform ";
  $sql .= ",:userAgent ";
  $sql .= ",:gekijo_id ";
  $sql .= ",:a_id ";
  $sql .= ",:r_id ";
  $sql .= ",:cookieData ";
  $sql .= ",:creation_dt ";
  $sql .= ",:modification_dt ";
  $sql .= ") ";
  $params = array(
   "requestUri" => $this->getsvr('REQUEST_URI'),
   "referer" => $this->getsvr('HTTP_REFERER'),
   "remoteAddr" => $remoteAddr,
   "remoteHost" => $this->realhost($remoteAddr),
   "browser" => $getBrowser["browser"],
   "version" => $getBrowser["version"],
   "platform" => $getBrowser["platform"],
   "userAgent" => $this->getsvr('HTTP_USER_AGENT'),
   "gekijo_id" => $gekijo_id,
   "a_id" => $a_id,
   "r_id" => $r_id,
   "cookieData" => $cookieData,
   "creation_dt" => $mynowtime,
   "modification_dt" => $mynowtime
  ) ;

  //ＳＱＬを実行
  $stmt = parent::execute($sql, $params);
  return $stmt;
  parent::closeStmt();
 }

 /*
  * @insertInvalidLog（TBL_INVALIDLOGテーブルにデータを追加する）
  * @引数：a_id　アンケートID
  * @引数：gekijo_id　劇場ID
  * @引数：r_id　アンケート場所
  * @引数：kind　不正理由
  * @返値：実行結果
  */
 public function insertInvalidLog($a_id, $gekijo_id, $r_id, $kind)
 {
  //更新日時
  $mynowtime = date ("Y-m-d H:i:s");
  $getBrowser = get_browser(null, true);
  $remoteAddr = $this->realaddr();

  //ＳＱＬ設定
  $sql = '';
  $sql .= 'INSERT INTO '.TBL_INVALIDLOG.' ';

  $sql .= "( ";
  $sql .= ' requestUri ';
  $sql .= ', referer ';
  $sql .= ', remoteAddr ';
  $sql .= ', remoteHost ';
  $sql .= ', browser ';
  $sql .= ', version ';
  $sql .= ', platform ';
  $sql .= ', userAgent ';
  $sql .= ', kind ';
  $sql .= ', gekijo_id ';
  $sql .= ', a_id ';
  $sql .= ', r_id ';
  $sql .= ', creation_dt ';
  $sql .= ") ";
  $sql .= "VALUES ";
  $sql .= "( ";
  $sql .= ":requestUri ";
  $sql .= ",:referer ";
  $sql .= ",:remoteAddr ";
  $sql .= ",:remoteHost ";
  $sql .= ",:browser ";
  $sql .= ",:version ";
  $sql .= ",:platform ";
  $sql .= ",:userAgent ";
  $sql .= ",:kind ";
  $sql .= ",:gekijo_id ";
  $sql .= ",:a_id ";
  $sql .= ",:r_id ";
  $sql .= ",:creation_dt ";
  $sql .= ") ";
  $params = array(
   "requestUri" => $this->getsvr('REQUEST_URI'),
   "referer" => $this->getsvr('HTTP_REFERER'),
   "remoteAddr" => $remoteAddr,
   "remoteHost" => $this->realhost($remoteAddr),
   "browser" => $getBrowser["browser"],
   "version" => $getBrowser["version"],
   "platform" => $getBrowser["platform"],
   "userAgent" => $this->getsvr('HTTP_USER_AGENT'),
   "kind" => $kind,
   "gekijo_id" => $gekijo_id,
   "a_id" => $a_id,
   "r_id" => $r_id,
   "creation_dt" => $mynowtime
  ) ;

  //ＳＱＬを実行
  $stmt = parent::execute($sql, $params);
  return $stmt;
  parent::closeStmt();
 }

 /*
  * @updateAccessLog（TBL_ACCESSLOGテーブルのデータを更新する）
  * @説明：アクセス時の更新は訪問回数のみ、回答失敗時、回答完了時は別途！・・・・・・
  * @引数：cookieData　クッキーデータ
  * @引数：visitCount　訪問回数
  * @返値：実行結果
  */
 public function updateAccessLog($cookieData, $visitCount)
 {
  //更新日時
  $mynowtime = date ("Y-m-d H:i:s");

  //ＳＱＬ設定
  $sql = '';
  $sql .= 'UPDATE '.TBL_ACCESSLOG.' SET ';
  $sql .= "visitCount = :visitCount";
  $sql .= ", modification_dt = :modification_dt";
  $sql .= " WHERE cookieData = :cookieData";

  $params = array(
   "visitCount" => $visitCount, //訪問回数
   "modification_dt" => $mynowtime, //最終更新日
   "cookieData" => $cookieData
  ) ;

  $stmt = parent::execute($sql, $params);
  return $stmt;
  parent::closeStmt();
 }

 /*
  * @updateErrorCount（TBL_ACCESSLOGテーブルのデータを更新する）
  * @説明：アクセス時の更新は回答失敗回数のみ、回答完了時は別途！・・・・・・
  * @引数：cookieData　クッキーデータ
  * @引数：errorCount　回答失敗回数
  * @返値：実行結果
  */
 public function updateErrorCount($cookieData, $errorCount)
 {
  //ＳＱＬ設定
  $sql = '';
  $sql .= 'UPDATE '.TBL_ACCESSLOG.' SET ';
  $sql .= "errorCount = :errorCount";
  $sql .= " WHERE cookieData = :cookieData";

  $params = array(
   "errorCount" => $errorCount, //回答失敗回数
   "cookieData" => $cookieData
  ) ;

  $stmt = parent::execute($sql, $params);
  return $stmt;
  parent::closeStmt();
 }

 /*
  * @updateKind（TBL_ACCESSLOGテーブルのデータを更新する）
  * @説明：アクセス時の更新は回答失敗、期限切れ、完了時
  * @引数：cookieData　クッキーデータ
  * @引数：kind　回答属性
  * @返値：実行結果
  */
 public function updateKind($cookieData, $kind)
 {
  //更新日時
  $mynowtime = date ("Y-m-d H:i:s");

  //ＳＱＬ設定
  $sql = '';
  $sql .= 'UPDATE '.TBL_ACCESSLOG.' SET ';

  switch($kind) {
   case 1 :
    $sql .= "kind = :kind";
    $sql .= ", errorCount = errorCount + 1";
    $sql .= " WHERE cookieData = :cookieData";

    $params = array(
     "kind" => $kind,  //属性 0:未回答 1:回答失敗 2:回答期限切れ 3:回答済み
     "cookieData" => $cookieData
    ) ;
    break ;
   case 2 :
   case 3 :
    $sql .= "kind = :kind";
    $sql .= ", answered_dt = :answered_dt";
    $sql .= " WHERE cookieData = :cookieData";

    $params = array(
     "kind" => $kind,  //属性 0:未回答 1:回答失敗 2:回答期限切れ 3:回答済み
     "answered_dt" => $mynowtime, //回答日日
     "cookieData" => $cookieData
    ) ;
    break ;
   default:
    return false;
    break;
  }

  $stmt = parent::execute($sql, $params);
  return $stmt;
  parent::closeStmt();
 }


 /*
  * @createAccessLogtbl（TBL_ACCESSLOGテーブルを作成する）
  * @引数：なし
  * @返値：実行結果
  */
 public function createAccessLogtbl( $sql = null )
 {

  $sql = "";
  $sql .= "create table IF NOT EXISTS ".TBL_ACCESSLOG." ";
  $sql .= "( ";
  $sql .= " id INT NOT NULL AUTO_INCREMENT primary key"; //ID
  // アクセス情報エリア
  $sql .= ", requestUri varchar(150) not null default ''"; //ページにアクセスするために指定された URI
  $sql .= ", referer varchar(150) not null default ''"; //現在のページに遷移する前に参照していたページのアドレス
  $sql .= ", remoteAddr varchar(150) default null"; //ユーザのIPアドレス
  $sql .= ", remoteHost varchar(150) not null default ''"; //ユーザのホスト名
  $sql .= ", browser varchar(150) not null default ''"; //ユーザのブラウザ名
  $sql .= ", version varchar(150) not null default ''"; //ユーザのブラウザバージョン
  $sql .= ", platform varchar(150) not null default ''"; //ユーザのプラットフォーム
  $sql .= ", userAgent varchar(150) not null default ''"; //ユーザーエージェント
  $sql .= ", kind int not null default 0"; //属性 0:未回答 1:回答失敗 2:回答期限切れ 3:回答済み
  $sql .= ", gekijo_id varchar(150) not null default ''"; //劇場ID
  $sql .= ", a_id varchar(150) not null default ''"; //アンケートID
  $sql .= ", r_id varchar(150) not null default ''"; //アンケート場所
  $sql .= ", cookieData varchar(150) not null default ''"; //クッキーデータ
  $sql .= ", visitCount int not null default 1"; //訪問回数
  $sql .= ", errorCount int not null default 0"; //回答失敗回数
  // 回答者・連絡先
  $sql .= ", user_id varchar(150) not null default ''"; //回答者ユーザーID
  $sql .= ", user_name varchar(150) not null default ''"; //回答者ユーザー名
  $sql .= ", user_tel varchar(150) not null default ''"; //回答者電話番号
  $sql .= ", user_mail varchar(150) not null default ''"; //回答者メールアドレス
  $sql .= ", user_postcode varchar(10) not null default ''"; //郵便番号
  $sql .= ", user_prefectures varchar(150) not null default ''"; //都道府県名
  $sql .= ", user_address1 varchar(150) not null default ''"; //市区町村名
  $sql .= ", user_address2 varchar(150) not null default ''"; //以降番地まで
  $sql .= ", user_address3 varchar(150) not null default ''"; //建物名など
  // 自動更新エリア
  $sql .= ", creation_dt datetime NOT NULL DEFAULT CURRENT_TIMESTAMP"; //作成日
  $sql .= ", modification_dt datetime not null default 0"; //最終訪問日
  $sql .= ", answered_dt datetime not null default 0"; //回答日
  $sql .= ", index cookieData_index (cookieData)"; //インデックス
  $sql .= ")  ENGINE=InnoDB DEFAULT CHARSET=utf8;";
  //ＳＱＬを実行
  $id = parent::runSql( $sql );
  //最後に操作したデータのIDを返す
  return $id;
 }

 /*
  * @createInvalidLogtbl（TBL_INVALIDLOGテーブルを作成する）
  * @引数：なし
  * @返値：実行結果
  */
 public function createInvalidLogtbl( $sql = null )
 {

  $sql = "";
  $sql .= "create table IF NOT EXISTS ".TBL_INVALIDLOG." ";
  $sql .= "( ";
  $sql .= " id INT NOT NULL AUTO_INCREMENT primary key"; //ID
  // アクセス情報エリア
  $sql .= ", requestUri varchar(150) not null default ''"; //ページにアクセスするために指定された URI
  $sql .= ", referer varchar(150) not null default ''"; //現在のページに遷移する前に参照していたページのアドレス
  $sql .= ", remoteAddr varchar(150) default null"; //ユーザのIPアドレス
  $sql .= ", remoteHost varchar(150) not null default ''"; //ユーザのホスト名
  $sql .= ", browser varchar(150) not null default ''"; //ユーザのブラウザ名
  $sql .= ", version varchar(150) not null default ''"; //ユーザのブラウザバージョン
  $sql .= ", platform varchar(150) not null default ''"; //ユーザのプラットフォーム
  $sql .= ", userAgent varchar(150) not null default ''"; //ユーザーエージェント
  $sql .= ", kind int not null default 0"; //理由 0:不正アクセス 1:クッキー設定不可 2:回答期限済 3:回答済み
  $sql .= ", gekijo_id varchar(150) not null default ''"; //劇場ID
  $sql .= ", a_id varchar(150) not null default ''"; //アンケートID
  $sql .= ", r_id varchar(150) not null default ''"; //アンケート場所
  // 回答者・連絡先
  $sql .= ", user_id varchar(150) not null default ''"; //回答者ユーザーID
  $sql .= ", user_name varchar(150) not null default ''"; //回答者ユーザー名
  $sql .= ", user_tel varchar(150) not null default ''"; //回答者電話番号
  $sql .= ", user_mail varchar(150) not null default ''"; //回答者メールアドレス
  // 自動更新エリア
  $sql .= ", creation_dt datetime NOT NULL DEFAULT CURRENT_TIMESTAMP"; //作成日
  $sql .= ", index a_id_index (gekijo_id, a_id)"; //インデックス
  $sql .= ")  ENGINE=InnoDB DEFAULT CHARSET=utf8;";
  //ＳＱＬを実行
  $id = parent::runSql( $sql );
  //最後に操作したデータのIDを返す
  return $id;
 }

 /**
  * UUID 生成
  * @return string
  * @throws \Exception
  */
 public static function generate(): string
 {
  $chars = str_split(self::PATTERN);

  foreach ($chars as $i => $char) {
   if ($char === 'x') {
    $chars[$i] = dechex(random_int(0, 15));
   } elseif ($char === 'y') {
    $chars[$i] = dechex(random_int(8, 11));
   }
  }

  return implode('', $chars);
 }

 /**
  * 生ＩＰアドレスを取得
  * @return string
  */
 function realaddr()
 {
  $ip = array();

  if (preg_match('/^\d+(?:\.\d+){3}$/D', $this->getsvr('HTTP_SP_HOST')))
   $ip[] = $_SERVER['HTTP_SP_HOST'];
  if (preg_match('/.*\s(\d+(?:\.\d+){3})/', $this->getsvr('HTTP_VIA'), $match))
   $ip[] = $match[1];
  if (preg_match('/^\d+(?:\.\d+){3}/', $this->getsvr('HTTP_CLIENT_IP'), $match))
   $ip[] = $match[0];
  if (preg_match('/^([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})/i', $this->getsvr('HTTP_CLIENT_IP'), $match))
   $ip[] = implode('.', array(hexdec($match[1]), hexdec($match[2]), hexdec($match[3]), hexdec($match[4])));
  if (preg_match('/.*\s(\d+(?:\.\d+){3})/', $this->getsvr('HTTP_FORWARDED'), $match))
   $ip[] = $match[1];
  if (preg_match('/^\d+(?:\.\d+){3}/', $this->getsvr('HTTP_X_FORWARDED_FOR'), $match))
   $ip[] = $match[0];
  if (preg_match('/^\d+(?:\.\d+){3}$/D', $this->getsvr('HTTP_FROM')))
   $ip[] = $_SERVER['HTTP_FROM'];

  $addr = '';
  foreach ($ip as $value)
   if (!preg_match('/^(?:10|172\.16|192\.168|127\.0|0\.|169\.254)\./', $value) and $addr=$value) break;

  return($addr ? $addr : $_SERVER['REMOTE_ADDR']);
 }
 /**
  * サーバデータを取得
  * @return string
  */
 function getsvr($key) {
   return(isset($_SERVER[$key]) ? $_SERVER[$key] : '');
 }

 /**
  * ホスト名を取得、ホスト名がない場合にはＩＰアドレスを返す
  * @return string
  */
 function realhost($addr="")
 {
  if (empty($addr)) return '';
  $host = gethostbyaddr($addr);
  if (empty($host)) $host = isset($_SERVER['REMOTE_HOST']) ? $_SERVER['REMOTE_HOST'] : '';

  return($host ? $host : $addr);
 }

/**
 * 
 * 正規表現でUserAgent分岐/OS名取得
 * 
 * @param string $user_agent
 * @return string
 */
 function getOs($user_agent = '')
 {
  if (empty($user_agent)) {
   // ユーザエージェント
   $user_agent = $_SERVER['HTTP_USER_AGENT'];
  }

  if (preg_match('/Windows NT 10.0/', $user_agent)) {
   $os = 'Windows 10';
  } elseif (preg_match('/Windows NT 6.3/', $user_agent)) {
   $os = 'Windows 8.1 / Windows Server 2012 R2';
  } elseif (preg_match('/Windows NT 6.2/', $user_agent)) {
   $os = 'Windows 8 / Windows Server 2012';
  } elseif (preg_match('/Windows NT 6.1/', $user_agent)) {
   $os = 'Windows 7 / Windows Server 2008 R2';
  } elseif (preg_match('/Windows NT 6.0/', $user_agent)) {
   $os = 'Windows Vista / Windows Server 2008';
  } elseif (preg_match('/Windows NT 5.2/', $user_agent)) {
   $os = 'Windows XP x64 Edition / Windows Server 2003';
  } elseif (preg_match('/Windows NT 5.1/', $user_agent)) {
   $os = 'Windows XP';
  } elseif (preg_match('/Windows NT 5.0/', $user_agent)) {
   $os = 'Windows 2000';
  } elseif (preg_match('/Windows NT 4.0/', $user_agent)) {
   $os = 'Microsoft Windows NT 4.0'; 
  } elseif (preg_match('/Mac OS X ([0-9\._]+)/', $user_agent, $matches)) {
   $os = 'Macintosh Intel ' . str_replace('_', '.', $matches[1]);
  } elseif (preg_match('/OS ([a-z0-9_]+)/', $user_agent, $matches)) {
   $os = 'iOS ' . str_replace('_', '.', $matches[1]);
  } elseif (preg_match('/Android ([a-z0-9\.]+)/', $user_agent, $matches)) {
   $os = 'Android ' . $matches[1];
  } elseif (preg_match('/Linux ([a-z0-9_]+)/', $user_agent, $matches)) {
   $os = 'Linux ' . $matches[1];
  } else {
   $os = 'unidentified';
  }

  return $os;
 }

}

?>