<?php
/*
 * ：ユーザーの所属劇場を管理するクラス
 * @親クラス  ：  ・pdodb
 */
require_once (dirname(__FILE__).'/pdoDb.class.php');
require_once (dirname(__FILE__).'/../setting.php');
require_once (dirname(__FILE__).'/dbconfig.php');

class gekijoDb extends pdodb
{

 /*
  * @selectGekijo2（TBL_GEKIJOテーブルのデータを抽出する）
  * @引数：parm1　劇場種別
  * @引数：parm2  劇場状態
  * @引数：parm3　劇場ID
  * @返値：抽出データ
  */
 public function selectGekijo2($parm1, $parm2, $parm3 = null)
 {
  //ＳＱＬ設定
  $sql = '';
  $sql .= 'SELECT * ';
  $sql .= 'FROM '.TBL_GEKIJO.' ';
  $sql .= "WHERE gekijo_kind >= :gekijo_kind ";
  $sql .= ' AND gekijo_existence <= :gekijo_existence ';

  $params = array(
   'gekijo_kind' => $parm1,
   'gekijo_existence' => $parm2
  );

  //パラメータの有無を判断（あればパラメータをバインド変数として実行）
  if ( ($parm3 != "") &&  ($parm3 != null) && !empty( $parm3 ) ){
   $sql .= "AND gekijo_id = :gekijo_id ";
   $params += array(
    'gekijo_id' => $parm3
   );
  }
  $sql .= 'ORDER BY gekijo_kind desc ';

  //ＳＱＬを実行及び返値をこのメソッドの返値として設定
  $stmt = parent::execute($sql, $params);
  $ret = $stmt->fetchAll();
  return $ret;
 }


 /*
  * @selectGekijo（TBL_GEKIJOテーブルのデータを抽出する）
  * @引数：parm　劇場ID
  * @返値：抽出データ
  */
 public function selectGekijo($parm = null)
 {

  //ＳＱＬ設定
  $sql = '';
  $sql .= 'SELECT * ';
  $sql .= 'FROM '.TBL_GEKIJO.' ';

  //パラメータの有無を判断（あればパラメータをバインド変数として実行）
  if ( $parm != null && !empty( $parm ) ){
   $sql .= "WHERE  gekijo_id = :gekijo_id";
   $params = array(
   'gekijo_id' => $parm
   );

   $stmt = parent::execute($sql, $params);
  }
  else {
   $stmt = parent::execute($sql);
  }

  //ＳＱＬを実行及び返値をこのメソッドの返値として設定
  $ret = $stmt->fetchAll();
  return $ret;
 }


 /*
  * @insertGekijo（TBL_GEKIJOテーブルにデータを追加する）
  * @引数：gekijo_id　劇場ID　他データ
  * @返値：実行結果
  */
 public function insertGekijo($gekijo_id, $gekijo_kind, $gekijo_existence, $gekijo_name, $gekijo_sname, $gekijo_explanation, $gekijo_postcode, $gekijo_prefectures, $gekijo_address1, $gekijo_address2, $gekijo_address3, $gekijo_url, $gekijo_tel, $gekijo_mail, $gekijo_comment, $user_id, $creation_user, $latitude, $longitude)
 {
  //セッション有効期限１時間
  $mysqltime = date ("Y-m-d H:i:s",strtotime(LOGIN_EXPIRATION));

  //ＳＱＬ設定
  $sql = '';
  $sql .= 'INSERT INTO '.TBL_GEKIJO.' ';
  $sql .= "( ";
  $sql .= ' gekijo_id ';
  $sql .= ', gekijo_kind ';
  $sql .= ', gekijo_existence ';
  $sql .= ', gekijo_name ';
  $sql .= ', gekijo_sname ';
  $sql .= ', gekijo_explanation ';
  $sql .= ', gekijo_postcode ';
  $sql .= ', gekijo_prefectures ';
  $sql .= ', gekijo_address1 ';
  $sql .= ', gekijo_address2 ';
  $sql .= ', gekijo_address3 ';
  $sql .= ', gekijo_url ';
  $sql .= ', gekijo_tel ';
  $sql .= ', gekijo_mail ';
  $sql .= ', gekijo_comment ';
  $sql .= ', user_id ';
  $sql .= ', creation_user ';
  $sql .= ', location ';
  $sql .= ") ";
  $sql .= "VALUES ";
  $sql .= "( ";
  $sql .= ":gekijo_id ";
  $sql .= ",:gekijo_kind ";
  $sql .= ",:gekijo_existence ";
  $sql .= ",:gekijo_name ";
  $sql .= ",:gekijo_sname ";
  $sql .= ",:gekijo_explanation ";
  $sql .= ",:gekijo_postcode ";
  $sql .= ",:gekijo_prefectures ";
  $sql .= ",:gekijo_address1 ";
  $sql .= ",:gekijo_address2 ";
  $sql .= ",:gekijo_address3 ";
  $sql .= ",:gekijo_url ";
  $sql .= ",:gekijo_tel ";
  $sql .= ",:gekijo_mail ";
  $sql .= ",:gekijo_comment ";
  $sql .= ",:user_id ";
  $sql .= ",:creation_user ";
  $sql .= ",GeomFromText(:location) ";
  $sql .= ") ";
  $params = array(
   "gekijo_id" => $gekijo_id,
   "gekijo_kind" => $gekijo_kind,
   "gekijo_existence" => $gekijo_existence,
   "gekijo_name" => $gekijo_name,
   "gekijo_sname" => $gekijo_sname,
   "gekijo_explanation" => $gekijo_explanation,
   "gekijo_postcode" => $gekijo_postcode,
   "gekijo_prefectures" => $gekijo_prefectures,
   "gekijo_address1" => $gekijo_address1,
   "gekijo_address2" => $gekijo_address2,
   "gekijo_address3" => $gekijo_address3,
   "gekijo_url" => $gekijo_url,
   "gekijo_tel" => $gekijo_tel,
   "gekijo_mail" => $gekijo_mail,
   "gekijo_comment" => $gekijo_comment,
   "user_id" => $user_id,
   "creation_user" => $creation_user,
   "location" => "POINT(" . $latitude . " " . $longitude . ")"
  ) ;

  //ＳＱＬを実行
  $stmt = parent::execute($sql, $params);
  return $stmt;
  parent::closeStmt();
 }


 /*
  * @updateGekijo（TBL_GEKIJOテーブルのデータを更新する）
  * @引数：gekijo_id　劇場ID　他データ
  * @返値：実行結果
  */
 public function updateGekijo($gekijo_id, $gekijo_kind, $gekijo_existence, $gekijo_name, $gekijo_sname, $gekijo_explanation, $gekijo_postcode, $gekijo_prefectures, $gekijo_address1, $gekijo_address2, $gekijo_address3, $gekijo_url, $gekijo_tel, $gekijo_mail, $gekijo_comment, $user_id, $modification_user, $latitude, $longitude)
 {
  //更新日時
  $mynowtime = date ("Y-m-d H:i:s");

  //ＳＱＬ設定
  $sql = '';
  $sql .= 'UPDATE '.TBL_GEKIJO.' SET ';
  $sql .= "gekijo_kind = :gekijo_kind";
  $sql .= ", gekijo_existence = :gekijo_existence";
  $sql .= ", gekijo_name = :gekijo_name";
  $sql .= ", gekijo_sname = :gekijo_sname";
  $sql .= ", gekijo_explanation = :gekijo_explanation";
  $sql .= ", gekijo_postcode = :gekijo_postcode";
  $sql .= ", gekijo_prefectures = :gekijo_prefectures";
  $sql .= ", gekijo_address1 = :gekijo_address1";
  $sql .= ", gekijo_address2 = :gekijo_address2";
  $sql .= ", gekijo_address3 = :gekijo_address3";
  $sql .= ", gekijo_url = :gekijo_url";
  $sql .= ", gekijo_tel = :gekijo_tel";
  $sql .= ", gekijo_mail = :gekijo_mail";
  $sql .= ", gekijo_comment = :gekijo_comment";
  $sql .= ", user_id = :user_id";
  $sql .= ", modification_dt = :modification_dt";
  $sql .= ", modification_user = :modification_user";
  $sql .= ", location = GeomFromText(:location)";
  $sql .= " WHERE gekijo_id = :gekijo_id";

  $params = array(
   "gekijo_kind" => $gekijo_kind,
   "gekijo_existence" => $gekijo_existence,
   "gekijo_name" => $gekijo_name,
   "gekijo_sname" => $gekijo_sname,
   "gekijo_explanation" => $gekijo_explanation,
   "gekijo_postcode" => $gekijo_postcode,
   "gekijo_prefectures" => $gekijo_prefectures,
   "gekijo_address1" => $gekijo_address1,
   "gekijo_address2" => $gekijo_address2,
   "gekijo_address3" => $gekijo_address3,
   "gekijo_url" => $gekijo_url,
   "gekijo_tel" => $gekijo_tel,
   "gekijo_mail" => $gekijo_mail,
   "gekijo_comment" => $gekijo_comment,
   "user_id" => $user_id,
   "modification_dt" => $mynowtime,
   "modification_user" => $modification_user,
   "location" => "POINT(" . $latitude . " " . $longitude . ")",
   "gekijo_id" => $gekijo_id
  ) ;

  //ＳＱＬを実行
  $stmt = parent::execute($sql, $params);
  return $stmt;
  parent::closeStmt();
 }


 /*
  * @updateExistence（TBL_GEKIJOテーブルのデータを劇場状態のみ更新する）
  * @引数：gekijo_id　劇場ID
  * @引数：gekijo_existence　劇場状態
  * @返値：実行結果
  */
 public function updateExistence($gekijo_id, $gekijo_existence)
 {

  //ＳＱＬ設定
  $sql = '';
  $sql .= 'UPDATE '.TBL_GEKIJO.' SET ';
  $sql .= " gekijo_existence = :gekijo_existence ";
  $sql .= " WHERE gekijo_id = :gekijo_id";
  $params = array(
   "gekijo_existence" => $gekijo_existence,
   "gekijo_id" => $gekijo_id
  ) ;

  //ＳＱＬを実行
  $stmt = parent::execute($sql, $params);
  return $stmt;
  parent::closeStmt();
 }


 /*
  * @deleteGekijo（TBL_GEKIJOテーブルのデータを削除する）
  * @引数：gekijo_id　劇場ID
  * @返値：実行結果
  */
 public function deleteGekijo($gekijo_id)
 {
  //ＳＱＬ設定
  $sql = '';
  $sql .= 'DELETE FROM '.TBL_GEKIJO.' ';
  $sql .= " WHERE gekijo_id = :gekijo_id";
  $params = array(
   "gekijo_id" => $gekijo_id
  ) ;

  //ＳＱＬを実行
  $stmt = parent::execute($sql, $params);
  return $stmt;
  parent::closeStmt();
 }

 /*
  * @createGekijotbl（TBL_GEKIJOテーブルを作成する）
  * @引数：なし
  * @返値：実行結果
  */
 public function createGekijotbl( $sql = null )
 {

  $sql = "";
  $sql .= "create table IF NOT EXISTS ".TBL_GEKIJO." ";
  $sql .= "( ";
  $sql .= " id INT NOT NULL AUTO_INCREMENT primary key"; //ID
  $sql .= ", gekijo_id varchar(4) unique not null default ''"; //劇場ID
  $sql .= ", gekijo_kind int default 0"; //劇場種別　0:システム管理者（メンテ用）　1:管理者　2:劇場管理者
  $sql .= ", gekijo_existence int default 0"; //劇場状態　0:有効  1:停止　2:削除
  $sql .= ", gekijo_expiration datetime not null default 0"; //劇場の有効期限
  // アンケート表示用情報　劇場で編集可能
  $sql .= ", gekijo_name varchar(150)  not null default ''"; //劇場正式名称
  $sql .= ", gekijo_sname varchar(150)  not null default ''"; //劇場名略称
  $sql .= ", gekijo_explanation varchar(150) default null"; //劇場説明文
  $sql .= ", gekijo_postcode varchar(10) not null default ''"; //郵便番号
  $sql .= ", gekijo_prefectures varchar(150) not null default ''"; //都道府県名
  $sql .= ", gekijo_address1 varchar(150) not null default ''"; //市区町村名
  $sql .= ", gekijo_address2 varchar(150) not null default ''"; //以降番地まで
  $sql .= ", gekijo_address3 varchar(150) not null default ''"; //建物名など
  $sql .= ", gekijo_url varchar(150) not null default ''"; //劇場サイトURL
  $sql .= ", gekijo_tel varchar(150) not null default ''"; //電話番号
  $sql .= ", gekijo_mail varchar(150) not null default ''"; //メールアドレス
  $sql .= ", gekijo_comment varchar(150) default null"; //自由記述コメント
  // 担当者・連絡先
  $sql .= ", user_id varchar(150) not null default ''"; //担当者ユーザーID
  $sql .= ", user_name varchar(150) not null default ''"; //担当者ユーザー名
  $sql .= ", user_tel varchar(150) not null default ''"; //電話番号
  $sql .= ", user_mail varchar(150) not null default ''"; //メールアドレス
  // 自動更新エリア
  $sql .= ', location geometry';
  $sql .= ", creation_dt datetime NOT NULL DEFAULT CURRENT_TIMESTAMP"; //作成日
  $sql .= ", creation_user varchar(150) default null"; //作成者
  $sql .= ", modification_dt datetime not null default 0"; //最終更新日
  $sql .= ", modification_user varchar(150) default null"; //最終更新者
  $sql .= ")  ENGINE=InnoDB DEFAULT CHARSET=utf8;";
  //ＳＱＬを実行
  $id = parent::runSql( $sql );
  //最後に操作したデータのIDを返す
  return $id;
 }


}

?>