<?php

/*
* @ＰＤＯラッパー  ：  よく使用する機能をまとめて簡素化する
* @メンバ          ：  ・$dbh →ＤＢハンドル
*                  ：  ・$stmt →ＳＱＬステートメント
*                  ：
* @機能            ：  ・__construct →ＤＢへのコネクションを確立する
*                  ：  ・closeDetaBase→ＤＢコネクションを閉じる
*                  ：  ・closeStmt →ステートメントを開放する
*                  ：  ・runSql →SQLを実行する
*                  ：  ・getDeta →DBからデータを抽出して連想配列で返す
*/
require_once (dirname(__FILE__).'/../setting.php');
require_once (dirname(__FILE__).'/dbconfig.php');

class pdodb
{
 protected $hasActiveTransaction;
 protected $dbh; //ＤＢハンドル
 protected $stmt; //ステートメント
 /*
  * @コンストラクタ（ＤＢハンドルに確立したコネクションをセットする）
  * @引数：$dbType:ＤＢの種類（Oracle，MySQL，PostgreSQL等）
  *      ：$dbHost:ホスト名もしくはＩＰ
  *      ：$dbName:ＤＢ名
  *      ：$dbUser:ＤＢユーザ
  *      ：$dbPass:ＤＢユーザのパスワード
  * @返値：なし
  */
 public function __construct()
 {
  $this->hasActiveTransaction = false; // autocommit:ON だからtrueなのだろう

  //DBへの接続
  try
  {
   //コネクションの確立
   $this->dbh = new PDO(
    "mysql:host=" . DB_HOST . ";dbname=" . DB_NAME , 
    DB_USER , 
    DB_PASSWORD , 
    array( PDO::MYSQL_ATTR_INIT_COMMAND => "SET CHARACTER SET `utf8`" )
   );
   $this->dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
   //トランザクションを初期化する
   $this->dbh->beginTransaction();
  }
  catch( PDOException $e )
  {
   if(DEBUG_LEVEL != "0"){
    error_log("[". date('Y-m-d H:i:s') . "]". "DB接続エラー\n","3","debug.log");
    error_log(print_r($e->getMessage(),true)."\n","3","debug.log");
    print('<font color="red">DB接続エラー : dbconfig.phpの設定を確認してください。</font></br>'.$e->getMessage());
   }
   exit;
  }
 }
 /*
  * @closeDataBase（ＤＢハンドルのコネクション（メンバ）を廃棄する）
  * @引数：なし
  * @返値：なし
  */
 public function closeDetaBase()
 {
  $this->dbh = null;
 }
 /*
  * closeStmt（ＳＱＬステートメント（メンバ）を廃棄する）
  * @引数：なし
  * @返値：なし
  */
 public function closeStmt()
 {
  $this->stmt->closeCursor;
  $this->stmt = null;
 }
 /*
  * @runSql（引数のＳＱＬを実行する。パラメータがある場合はバインドを設定後）
  * @引数：$sql :SQL文
  * 　　：$params:パラメータ（デフォルトはNULL）
  * @返値：操作したデータのID
  */
 public function runSql( $sql , $params = null )
 {
  $id = 0;
  try
  {
   //SQLをステートメントにセットする
   $this->stmt = $this->dbh->prepare( $sql );
   //パラメータの有無を判断（あればパラメータをバインド変数として実行）
   if ( empty( $params ) )
   {
    $result = $this->stmt->execute();
   }
   else
   {
    $result = $this->stmt->execute( $params );
   }
   //最後に操作したデータのIDを返す
   $id = $this->dbh->lastInsertId();
   //実行結果を評価
   if ( !( $result ) )
   {
    throw new Exception('SQLの実行が失敗しました');
   }
   //最後に操作したデータのIDを返す
   return $id;
  }
  catch( PDOException $e )
  {
   if(DEBUG_LEVEL != "0"){
    error_log("[". date('Y-m-d H:i:s') . "]". "PDOException\n","3","debug.log");
    error_log(print_r($e->getMessage(),true)."\n","3","debug.log");
    error_log("<<<sql>>>\n","3","debug.log");
    error_log(print_r($sql,true)."\n","3","debug.log");
    error_log("<<<params>>>\n","3","debug.log");
    error_log(print_r($params,true)."\n","3","debug.log");
   }
   return null;
   exit;
  }
 }
 /*
  * @getDeta（引数のＳＱＬを実行して抽出した結果を返す。
  *       パラメータがある場合はバインドを設定後）
  * @引数：$sql :SQL文
  *      ：$params:パラメータ（デフォルトはNULL）
  * @返値：操作したデータのID
  */
 public function getDeta( $sql , $params = null )
 {
  //SQLを実行する
  $this->runSql( $sql , $params );
  //実行した結果（連想配列）を返す
  return $this->stmt->fetchAll( PDO::FETCH_ASSOC );
 }

 public function fetchData()
 {
  //実行した結果（連想配列）を返す
  return $this->stmt->fetch( PDO::FETCH_ASSOC );
 }

 public function beginTransaction () {
  if ( $this->hasActiveTransaction ) {
   return false;
  } else {
  $this->hasActiveTransaction = $this->dbh->beginTransaction ();
  return $this->hasActiveTransaction;
  }
 }

 public function commit () {
  $this->dbh->commit();
  $this->hasActiveTransaction = false;
 }

 public function rollback () {
  $this->dbh->rollback ();
  $this->hasActiveTransaction = false;
 }

 /*
  * @execute（引数のＳＱＬを実行して抽出した結果を返す。
  *       パラメータがある場合はバインドを設定後）
  * @引数：$sql :SQL文（バインドパラメータを含めることが可能）
  *      ：$params_array:SQL文の中のバインドパラメータに代入する値またはその配列（可変長引数）
  * @返値：PDOStatementオブジェクト
  */
 public function execute($sql, ...$params_array) {
  try
  {
   //SQLをステートメントにセットする
   $this->stmt = $this->dbh->prepare( $sql );

   // ----- パラメータを一つの配列に統合 ----- //
   $integrated_params = array();
   $index = 1;

   foreach ($params_array as $params) {
    // $paramsがスカラまたはnullの時は配列に変換
    if (is_scalar($params) || $params === null) {
     $params = array($params);
    }

    foreach ($params as $param_id => $value) {
     // 数値添字のときは疑問符パラメータとみなす
     if (gettype($param_id) == 'integer') {
      $integrated_params[$index] = $value; // 疑問符パラメータ
     } else {
      $integrated_params[$param_id] = $value; // 名前付きパラメータ
     }

     $index++;
    }
   }

   // ----- データ型に応じてバインド ----- //
   foreach ($integrated_params as $param_id => $value) {
    switch (gettype($value)) {
     case 'boolean':
      $param_type = PDO::PARAM_BOOL;
      break;

     case 'integer':
      $param_type = PDO::PARAM_INT;
      break;

     case 'double':
      $param_type = PDO::PARAM_STR;
      break;

     case 'string':
      $param_type = PDO::PARAM_STR;
      break;

     case 'array':
      $value = json_encode($value);
      $param_type = PDO::PARAM_STR;
      break;

     case 'NULL':
      $param_type = PDO::PARAM_NULL;
      break;

     default:
      $param_type = PDO::PARAM_STR;

    }
    $this->stmt->bindValue($param_id, $value, $param_type);
   }
   $this->stmt->execute();
   return $this->stmt;

  }
  catch( PDOException $e )
  {
   if(DEBUG_LEVEL != "0"){
    error_log("[". date('Y-m-d H:i:s') . "]". "PDOException\n","3","debug.log");
    error_log(print_r($e->getMessage(),true)."\n","3","debug.log");
    error_log("<<<sql>>>\n","3","debug.log");
    error_log(print_r($sql,true)."\n","3","debug.log");
    error_log("<<<params_array>>>\n","3","debug.log");
    error_log(print_r($params_array,true)."\n","3","debug.log");
   }
   return null;
   exit;
  }

 }

}

?>